@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2016 Michal Čihař <michal@cihar.com>
+# Copyright (c) 2016 Michal Cihar <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -24,7 +24,7 @@ from urllib.parse import urlencode
 
 import json
 
-__version__ = '0.6'
+__version__ = '0.7'
 
 URL = 'https://weblate.org/'
 DEVEL_URL = 'https://github.com/WeblateOrg/wlc'
@@ -59,7 +59,8 @@ class Weblate(object):
                 )
             elif error.code == 404:
                 raise WeblateException(
-                    'Object not found on the server'
+                    'Object not found on the server '
+                    '(maybe operation is not supported on the server)'
                 )
             elif error.code == 403:
                 raise WeblateException(
@@ -69,7 +70,7 @@ class Weblate(object):
                 'HTTP error {0}: {1}'.format(error.code, error.reason)
             )
 
-    def request(self, path, params=None):
+    def request(self, path, params=None, raw=False):
         """Construct request object."""
         if not path.startswith('http'):
             path = '{0}{1}'.format(self.url, path)
@@ -88,6 +89,8 @@ class Weblate(object):
         except IOError as error:
             self.process_error(error)
             raise
+        if raw:
+            return content
         try:
             result = json.loads(content.decode('utf-8'))
         except ValueError:
@@ -110,7 +113,6 @@ class Weblate(object):
         """Wrapper for listing objects."""
         while path is not None:
             data = self.get(path)
-
             for item in data['results']:
                 yield parser(weblate=self, **item)
 
@@ -155,6 +157,10 @@ class Weblate(object):
         """List components in the instance."""
         return self.list_factory(path, Component)
 
+    def list_changes(self, path='changes/'):
+        """List components in the instance."""
+        return self.list_factory(path, Change)
+
     def list_translations(self, path='translations/'):
         """List translations in the instance."""
         return self.list_factory(path, Translation)
@@ -190,12 +196,18 @@ class LazyObject(dict):
     def _load_params(self, **kwargs):
         for param in self._params:
             if param in kwargs:
-                if param in self._mappings:
-                    self._data[param] = self._mappings[param](
-                        self.weblate, **kwargs[param]
-                    )
+                value = kwargs[param]
+                if value is not None and param in self._mappings:
+                    if isinstance(value, str):
+                        self._data[param] = self._mappings[param](
+                            self.weblate, url=value
+                        )
+                    else:
+                        self._data[param] = self._mappings[param](
+                            self.weblate, **value
+                        )
                 else:
-                    self._data[param] = kwargs[param]
+                    self._data[param] = value
                 del kwargs[param]
         for key in kwargs:
             self._attribs[key] = kwargs[key]
@@ -237,6 +249,7 @@ class LazyObject(dict):
 
     def to_value(self):
         """Return identifier for the object."""
+        self.ensure_loaded(self._id)
         return self.__getattr__(self._id)
 
 
@@ -251,6 +264,17 @@ class Language(LazyObject):
     _id = 'code'
 
 
+class LanguageStats(LazyObject):
+
+    """Language object."""
+
+    _params = (
+        'total', 'code', 'translated_words', 'language', 'translated',
+        'translated_percent', 'total_words', 'words_percent',
+    )
+    _id = 'code'
+
+
 class RepoMixin(object):
 
     """Repository mixin providing generic repository wide operations."""
@@ -280,6 +304,13 @@ class RepoMixin(object):
             operation='pull'
         )
 
+    def reset(self):
+        """Reset Weblate repository to upstream."""
+        return self.weblate.post(
+            self._get_repo_url(),
+            operation='reset'
+        )
+
 
 class ProjectRepository(LazyObject, RepoMixin):
 
@@ -336,6 +367,22 @@ class Project(LazyObject, RepoObjectMixin):
             self._attribs['components_list_url']
         )
 
+    def statistics(self):
+        """Return statistics for component."""
+        self.ensure_loaded('statistics_url')
+        url = self._attribs['statistics_url']
+        return [
+            LanguageStats(self.weblate, url, **item)
+            for item in self.weblate.get(url)
+        ]
+
+    def changes(self):
+        """List changes in the project."""
+        self.ensure_loaded('changes_list_url')
+        return self.weblate.list_changes(
+            self._attribs['changes_list_url']
+        )
+
 
 class Component(LazyObject, RepoObjectMixin):
 
@@ -391,6 +438,13 @@ class Component(LazyObject, RepoObjectMixin):
             self._get_lock_url(),
         )
 
+    def changes(self):
+        """List changes in the project."""
+        self.ensure_loaded('changes_list_url')
+        return self.weblate.list_changes(
+            self._attribs['changes_list_url']
+        )
+
 
 class Translation(LazyObject, RepoObjectMixin):
 
@@ -405,7 +459,7 @@ class Translation(LazyObject, RepoObjectMixin):
         'is_template', 'translated_percent', 'fuzzy_percent',
         'failing_checks_percent', 'last_change', 'last_author',
     )
-    _id = 'slug'
+    _id = 'language_code'
     _mappings = {
         'language': Language,
         'component': Component,
@@ -423,6 +477,24 @@ class Translation(LazyObject, RepoObjectMixin):
         data = self.weblate.get(self._attribs['statistics_url'])
         return Statistics(weblate=self.weblate, **data)
 
+    def changes(self):
+        """List changes in the project."""
+        self.ensure_loaded('changes_list_url')
+        return self.weblate.list_changes(
+            self._attribs['changes_list_url']
+        )
+
+    def download(self, convert=None):
+        """Download translation file from server."""
+        self.ensure_loaded('file_url')
+        url = self._attribs['file_url']
+        if convert is not None:
+            url = '{0}?{1}'.format(
+                url,
+                urlencode({'format': convert})
+            )
+        return self.weblate.request(url, raw=True)
+
 
 class Statistics(LazyObject):
 
@@ -433,3 +505,18 @@ class Statistics(LazyObject):
         'total_words', 'failing', 'translated_words', 'url_translate',
         'fuzzy_percent', 'translated', 'fuzzy', 'total', 'last_change', 'name',
     )
+
+
+class Change(LazyObject):
+
+    """Change object."""
+
+    _params = (
+        'url', 'unit', 'translation', 'component',
+        'timestamp', 'action_name', 'target',
+    )
+    _id = 'id'
+    _mappings = {
+        'translation': Translation,
+        'component': Component,
+    }
