@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2016 Michal Čihař <michal@cihar.com>
+# Copyright (c) 2016 Michal Cihar <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -275,6 +275,47 @@ class ObjectCommand(Command):
         """Main execution of the command."""
         raise NotImplementedError()
 
+    @staticmethod
+    def check_result(result, message):
+        """Check result json data."""
+        if not result['result']:
+            raise CommandError(
+                message,
+                result['detail'] if 'detail' in result else '',
+            )
+
+
+class ComponentCommand(ObjectCommand):
+
+    """Wrapper to allow only component objects."""
+
+    def get_object(self):
+        """Return component object."""
+        obj = super(ComponentCommand, self).get_object()
+        if not isinstance(obj, wlc.Component):
+            raise CommandError('Not supported')
+        return obj
+
+    def run(self):
+        """Main execution of the command."""
+        raise NotImplementedError()
+
+
+class TranslationCommand(ObjectCommand):
+
+    """Wrapper to allow only translation objects."""
+
+    def get_object(self):
+        """Return translation object."""
+        obj = super(TranslationCommand, self).get_object()
+        if not isinstance(obj, wlc.Translation):
+            raise CommandError('Not supported')
+        return obj
+
+    def run(self):
+        """Main execution of the command."""
+        raise NotImplementedError()
+
 
 @register_command
 class Version(Command):
@@ -356,7 +397,7 @@ class ListTranslations(Command):
 
 
 @register_command
-class ShowObject(ObjectCommand):
+class Show(ObjectCommand):
 
     """Show object."""
 
@@ -369,7 +410,7 @@ class ShowObject(ObjectCommand):
 
 
 @register_command
-class ListObject(ObjectCommand):
+class List(ObjectCommand):
 
     """List object."""
 
@@ -388,7 +429,7 @@ class ListObject(ObjectCommand):
 
 
 @register_command
-class CommitObject(ObjectCommand):
+class Commit(ObjectCommand):
 
     """Commit object."""
 
@@ -399,15 +440,11 @@ class CommitObject(ObjectCommand):
         """Executor."""
         obj = self.get_object()
         result = obj.commit()
-        if not result['result']:
-            raise CommandError(
-                'Failed to commit changes!',
-                result['detail'],
-            )
+        self.check_result(result, 'Failed to commit changes!')
 
 
 @register_command
-class PushObject(ObjectCommand):
+class Push(ObjectCommand):
 
     """Push object."""
 
@@ -421,15 +458,11 @@ class PushObject(ObjectCommand):
         """Executor."""
         obj = self.get_object()
         result = obj.push()
-        if not result['result']:
-            raise CommandError(
-                'Failed to push changes!',
-                result['detail'] if 'detail' in result else '',
-            )
+        self.check_result(result, 'Failed to push changes!')
 
 
 @register_command
-class PullObject(ObjectCommand):
+class Pull(ObjectCommand):
 
     """Pull object."""
 
@@ -443,15 +476,29 @@ class PullObject(ObjectCommand):
         """Executor."""
         obj = self.get_object()
         result = obj.pull()
-        if not result['result']:
-            raise CommandError(
-                'Failed to pull changes!',
-                result['detail'],
-            )
+        self.check_result(result, 'Failed to pull changes!')
 
 
 @register_command
-class RepoObject(ObjectCommand):
+class Reset(ObjectCommand):
+
+    """Reset object."""
+
+    name = 'reset'
+    description = (
+        "Resets all changes in Weblate repository to upstream "
+        "in translation, component or project"
+    )
+
+    def run(self):
+        """Executor."""
+        obj = self.get_object()
+        result = obj.reset()
+        self.check_result(result, 'Failed to reset changes!')
+
+
+@register_command
+class Repo(ObjectCommand):
 
     """Display repository status for object."""
 
@@ -468,7 +515,24 @@ class RepoObject(ObjectCommand):
 
 
 @register_command
-class StatsObject(ObjectCommand):
+class Changes(ObjectCommand):
+
+    """Display repository status for object."""
+
+    name = 'changes'
+    description = (
+        "Displays list of changes "
+        "for translation, component or project"
+    )
+
+    def run(self):
+        """Executor."""
+        obj = self.get_object()
+        self.print(list(obj.changes()))
+
+
+@register_command
+class Stats(ObjectCommand):
 
     """Display repository statistics for object."""
 
@@ -482,31 +546,15 @@ class StatsObject(ObjectCommand):
         """Executor."""
         obj = self.get_object()
         if isinstance(obj, wlc.Project):
-            raise CommandError('Not supported')
+            self.print(list(obj.statistics()))
         elif isinstance(obj, wlc.Component):
             self.print(list(obj.statistics()))
         else:
             self.print(obj.statistics())
 
 
-class ComponentCommand(ObjectCommand):
-
-    """Wrapper to allow only component objects."""
-
-    def get_object(self):
-        """Return component object."""
-        obj = super(ComponentCommand, self).get_object()
-        if not isinstance(obj, wlc.Component):
-            raise CommandError('Not supported')
-        return obj
-
-    def run(self):
-        """Main execution of the command."""
-        raise NotImplementedError()
-
-
 @register_command
-class LockStatusObject(ComponentCommand):
+class LockStatus(ComponentCommand):
 
     """Show lock status."""
 
@@ -522,7 +570,7 @@ class LockStatusObject(ComponentCommand):
 
 
 @register_command
-class LockObject(ComponentCommand):
+class Lock(ComponentCommand):
 
     """Lock component for transaltion."""
 
@@ -538,7 +586,7 @@ class LockObject(ComponentCommand):
 
 
 @register_command
-class UnlockObject(ComponentCommand):
+class Unlock(ComponentCommand):
 
     """Unock component for transaltion."""
 
@@ -553,13 +601,43 @@ class UnlockObject(ComponentCommand):
         obj.unlock()
 
 
-def main(settings=None, stdout=None, args=None):
-    """Execution entry point."""
-    parser = get_parser()
-    if args is None:
-        args = sys.argv[1:]
-    args = parser.parse_args(args)
+@register_command
+class Download(TranslationCommand):
 
+    """Downloads translation file."""
+
+    name = 'download'
+    description = (
+        "Downloads translation file"
+    )
+
+    @classmethod
+    def add_parser(cls, subparser):
+        """Create parser for command line."""
+        parser = super(Download, cls).add_parser(subparser)
+        parser.add_argument(
+            '-c', '--convert',
+            help='Convert file format on server (defaults to none)'
+        )
+        parser.add_argument(
+            '-o', '--output',
+            help='File where to store output (defaults to stdout)'
+        )
+        return parser
+
+    def run(self):
+        """Executor."""
+        obj = self.get_object()
+        content = obj.download(self.args.convert)
+        if self.args.output and self.args.output != '-':
+            with open(self.args.output, 'wb') as handle:
+                handle.write(content)
+        else:
+            self.stdout.buffer.write(content)
+
+
+def parse_settings(args, settings):
+    """Read settings based on command line params."""
     config = WeblateConfig(args.config_section)
     if settings is None:
         config.load(args.config)
@@ -572,6 +650,18 @@ def main(settings=None, stdout=None, args=None):
         if value is not None:
             config.set(args.config_section, override, value)
 
+    return config
+
+
+def main(settings=None, stdout=None, args=None):
+    """Execution entry point."""
+    parser = get_parser()
+    if args is None:
+        args = sys.argv[1:]
+    args = parser.parse_args(args)
+
+    config = parse_settings(args, settings)
+
     command = COMMANDS[args.cmd](args, config, stdout)
     try:
         command.run()
