@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright © 2016 Michal Čihař <michal@cihar.com>
+# Copyright (c) 2016 Michal Cihar <michal@cihar.com>
 #
 # This file is part of Weblate Client <https://github.com/WeblateOrg/wlc>
 #
@@ -30,10 +30,11 @@ class ResponseHandler(object):
 
     """httpretty response handler."""
 
-    def __init__(self, body, filename):
+    def __init__(self, body, filename, auth=False):
         """Construct response handler object."""
         self.body = body
         self.filename = filename
+        self.auth = auth
 
     def get_filename(self, request):
         """Return filename for given request."""
@@ -60,10 +61,13 @@ class ResponseHandler(object):
 
     def __call__(self, request, uri, headers):
         """Function call interface for httpretty."""
+        if self.auth:
+            if request.headers['Authorization'] != 'Token KEY':
+                return (403, headers, '')
         return (200, headers, self.get_content(request))
 
 
-def register_uri(path, domain='http://127.0.0.1:8000/api'):
+def register_uri(path, domain='http://127.0.0.1:8000/api', auth=False):
     """Simplified URL registration."""
     filename = os.path.join(DATA_TEST_BASE, path.replace('/', '-'))
     url = '/'.join((domain, path, ''))
@@ -71,23 +75,30 @@ def register_uri(path, domain='http://127.0.0.1:8000/api'):
         httpretty.register_uri(
             httpretty.GET,
             url,
-            body=ResponseHandler(handle.read(), filename),
+            body=ResponseHandler(handle.read(), filename, auth),
             content_type='application/json'
         )
         httpretty.register_uri(
             httpretty.POST,
             url,
-            body=ResponseHandler(handle.read(), filename),
+            body=ResponseHandler(handle.read(), filename, auth),
             content_type='application/json'
         )
 
 
-def register_error(path, code, domain='http://127.0.0.1:8000/api'):
+def raise_error(request, uri, headers):
+    """Raise IOError."""
+    # pylint: disable=W0613
+    raise IOError('Some error')
+
+
+def register_error(path, code, domain='http://127.0.0.1:8000/api', body=None):
     """Simplified URL error registration."""
     url = '/'.join((domain, path, ''))
     httpretty.register_uri(
         httpretty.GET,
         url,
+        body=body,
         status=code
     )
 
@@ -95,18 +106,26 @@ def register_error(path, code, domain='http://127.0.0.1:8000/api'):
 def register_uris():
     """Register URIs for httpretty."""
     paths = (
+        'changes',
         'projects', 'components', 'translations',
         'projects/hello',
+        'projects/hello/changes',
+        'projects/hello/components',
+        'projects/hello/statistics',
+        'projects/empty',
+        'projects/empty/components',
         'projects/invalid',
         'components/hello/weblate',
         'components/hello/android',
         'translations/hello/weblate/cs',
         'projects/hello/repository',
         'components/hello/weblate/repository',
+        'components/hello/weblate/changes',
+        'translations/hello/weblate/cs/file',
         'translations/hello/weblate/cs/repository',
+        'translations/hello/weblate/cs/changes',
         'components/hello/weblate/statistics',
         'translations/hello/weblate/cs/statistics',
-        'projects/hello/components',
         'components/hello/weblate/translations',
         'components/hello/weblate/lock',
         'languages',
@@ -114,11 +133,14 @@ def register_uris():
     for path in paths:
         register_uri(path)
 
+    register_uri('projects/acl', auth=True)
+
     register_uri('projects', domain='https://example.net')
     register_error('projects/nonexisting', 404)
     register_error('projects/denied', 403)
     register_error('projects/throttled', 429)
     register_error('projects/error', 500)
+    register_error('projects/io', 500, body=raise_error)
 
 
 class APITest(TestCase):
